breed [ tokens token ]
breed [ agents agent ]

patches-own
[
 nrtokensneighbors 
]

agents-own
[
   t-count        ; count of the number of tokens collected, tokens represent resource units
   threshold      ; threshold when an agent harvest a token, based on the number of tokens on the 8 surrounding cells
   obsdens        ; observed density within the radius
   usedthreshold  ; the threshold used by the agent (can be different than 'threshold' when an agent makes and error)
   selfish        ; is 1 when an agent is selfish
]
   
globals [p initialamount nragents headings avgtokens avgthreshold]

to setup
   ca
   set p 0.01 ; regeneration rate of resource units
   set initialamount (max-pxcor + 1) * (max-pycor + 1) * 0.5
   set initialamount int initialamount
   set nragents 20
   set headings [0 90 180 270] ; agents can move one of four possible directions
   set avgtokens 0
   set avgthreshold 0
   setup-agents
   setup-tokens
   setup-plot
   set alpha2 2 * alpha1 - 1
   set beta2 2 * beta1 - 1
end

;; Populate the world with tokens.
to setup-tokens
  set-default-shape tokens "star"
  let teller 0
  while [teller < initialamount] [
    ask one-of patches 
    [
      if count turtles-here = 0 [sprout-tokens 1 [set color green] set teller teller + 1]
    ]
  ]
end

;; Populate the world with agents along the x-axis, spaced evenly.
to setup-agents
   set-default-shape agents "circle"

   create-agents nragents [setxy random-pxcor random-pycor]

   ask agents [
     set color yellow 
     set heading one-of headings
     set t-count 0   ;; Start with no tokens
     set threshold 4
     set obsdens 0
     ifelse who < egoists [set selfish 1 set threshold 0][set selfish 0]
   ]
   
end

to go
  move-agents
  do-plots
  grow-tokens
  if ticks > 900 [  ; for simulations of 1000 ticks, the last 100 ticks are used to derive summary statistics
    set avgtokens avgtokens + 0.01 * count tokens 
    set avgthreshold avgthreshold + 0.01 * sum [threshold] of agents / nragents
  ]
  tick
end

;; Calculates the total number of tokens collected by all agents.  Used for BehaviorSpace.
to-report total-tokens
  let t-tokens 0
  ask agents
  [
    set t-tokens (t-tokens + t-count)
  ]
  report t-tokens
end

;; Grow new tokens based on p*n/8.
to grow-tokens
  ask patches with [count turtles-on self = 0 ] ; sequential updating
  [
    set nrtokensneighbors count neighbors with [count tokens-on self > 0]
  ]
  ask patches with [count turtles-on self = 0]  ;; Grow tokens only on unoccupied cells (no tokens or agents)
  [
    if ((random-float 1) < (p * nrtokensneighbors / 8))  ;; Regeneration probability p*n/8
    [
      sprout-tokens 1 [set color green] 
    ]
  ]
end

;; Agents are limited to moving N, S, E, and W. Agents move a certain number of
;; cells per call to this function, depending on their speed. Each agent has a different desired speed, and every timestep, there is some variation in the amount
to move-agents

  ask agents [set usedthreshold 100]

  ask agents [
    if count tokens-here > 0 [  ; if there are tokens decide when to harvest
      ifelse sum [count tokens-here] of neighbors >= threshold or selfish = 1 [
        set usedthreshold sum [count tokens-here] of neighbors
        set t-count t-count + 1
        ask one-of tokens-here [die] 
      ][
        if random-float 1 < perror or selfish = 1 [
          set usedthreshold sum [count tokens-here] of neighbors
          set t-count t-count + 1
          ask one-of tokens-here [die]
        ]
      ]
    ]
  ]
  ask agents [  
    ; decide where to move to
    let possible-headings [0 90 180 270]
    let maxquad 0
    let headingmax 0
    let quad1 0
    let quad2 0
    let quad3 0
    let quad4 0
    let thisagent self
    let oldheading heading
    ; check for each direction the number of tokens within a vision couse of radius vision
    ask patch-at 0 1
    [
      ifelse count agents-here > 0 [set possible-headings remove 0 possible-headings]
      [
        ask thisagent [
          set heading 0
          let vistemp 1 
          while [vistemp < vision]
          [
            set quad1 quad1 + count tokens in-cone vistemp 90
            set vistemp vistemp + 1
          ]
        ]
      ]
    ]
    ask patch-at 1 0 
    [
      ifelse count agents-here > 0 [set possible-headings remove 90 possible-headings]
      [
        ask thisagent [
          set heading 90 
          let vistemp 1
          while [vistemp < vision]
          [
            set quad2 quad2 + count tokens in-cone vistemp 90
            set vistemp vistemp + 1
          ]
        ]
       ]
     ]

     ask patch-at 0 -1 
     [
       ifelse count agents-here > 0 [set possible-headings remove 180 possible-headings]
       [
         ask thisagent [
           set heading 180 
           let vistemp 1 
           while [vistemp < vision]
           [
             set quad3 quad3 + count tokens in-cone vistemp 90
             set vistemp vistemp + 1
           ]
         ]
       ]
     ]
     ask patch-at -1 0 
     [
       ifelse count agents-here > 0 [set possible-headings remove 270 possible-headings]
       [
         ask thisagent [
           set heading 270
           let vistemp 1
           while [vistemp < vision]
           [
             set quad4 quad4 + count tokens in-cone vistemp 90
             set vistemp vistemp + 1
           ]
         ]
       ]
      ]
      
      ; find the the direction with the highest number of tokens in vision. In this default version, we do not discount distance of the tokens but that could be a future direction
      ifelse quad2 > quad1 [set maxquad quad2][set maxquad quad1]
      if quad3 > maxquad [set maxquad quad3]
      if quad4 > maxquad [set maxquad quad4]
      
      let possible-maxheadings [] ; there might be more than one direction the maximum, and therefore the number of possible directions can be more than one.
      if maxquad = quad1 and member? 0 possible-headings [set possible-maxheadings lput 0 possible-maxheadings]
      if maxquad = quad2 and member? 90 possible-headings [set possible-maxheadings lput 90 possible-maxheadings]
      if maxquad = quad3 and member? 180 possible-headings [set possible-maxheadings lput 180 possible-maxheadings]
      if maxquad = quad4 and member? 270 possible-headings [set possible-maxheadings lput 270 possible-maxheadings]
      ; we assume that agents will not go back to the cell they came from 
      if oldheading = 0 [set possible-maxheadings remove 180 possible-maxheadings]
      if oldheading = 90 [set possible-maxheadings remove 270 possible-maxheadings]
      if oldheading = 180 [set possible-maxheadings remove 0 possible-maxheadings]
      if oldheading = 270 [set possible-maxheadings remove 90 possible-maxheadings]
      
      ifelse random-float 1 > pstraightmove [
        ; if agents make deliberate decisions and there are options, take randomly one of the options, otherwise continue the direction the agents was moving in
        ifelse length possible-maxheadings > 0 [set heading one-of possible-maxheadings][set heading oldheading] ]
      [set heading oldheading]
    fd 1
  ]
  
  ; update the thresholds based on the observations
  ask agents [
    set obsdens count tokens in-radius vision / count patches in-radius vision
    if selfish = 0 [ 
      ifelse obsdens < (threshold / 8) [ ; is the observed density is lower then threshold adjust the threshold
        set threshold threshold * alpha1 + (1 - alpha1) * obsdens * 8
      ][
        set threshold threshold * alpha2 + (1 - alpha2) * int (threshold + 0.5)
      ]
    ] 
  ]
  
  ask agents [
    let thisagent self
    let mythreshold threshold
    ask agents in-radius vision [
      let thatagent self
        ifelse [usedthreshold] of thatagent < mythreshold [
 
          ask thisagent [set threshold threshold * beta1 + (1 - beta1) * [usedthreshold] of thatagent]
        ][
          ask thisagent [set threshold threshold * beta2 + (1 - beta2) * int (threshold + 0.5)]
        ]
    ]
  ]
end

to do-plots
   set-current-plot "Tokens Remaining"
   plot count tokens
   
   set-current-plot "histogram"
   histogram [threshold] of agents
   
   set-current-plot "threshold"
   plot sum [threshold] of agents / nragents
end

to setup-plot
  set-current-plot "histogram"
  set-plot-x-range 0 8
  set-plot-y-range 0 count agents
  set-histogram-num-bars 10
end
@#$#@#$#@
GRAPHICS-WINDOW
167
11
417
282
-1
-1
8.0
1
10
1
1
1
0
1
1
1
0
29
0
29
1
1
1
ticks

BUTTON
1
10
56
43
NIL
Go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL

BUTTON
109
10
164
43
NIL
Setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

PLOT
418
12
696
235
Tokens Remaining
Seconds
Number of Tokens
0.0
10.0
0.0
10.0
true
false
PENS
"default" 1.0 0 -16777216 true

BUTTON
55
10
110
43
NIL
go
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

PLOT
944
10
1144
234
histogram
NIL
NIL
0.0
10.0
0.0
10.0
true
false
PENS
"default" 1.0 1 -16777216 true

SLIDER
2
44
164
77
vision
vision
0
10
3
1
1
NIL
HORIZONTAL

SLIDER
2
79
164
112
alpha1
alpha1
0
1
0.9
0.01
1
NIL
HORIZONTAL

PLOT
697
12
944
234
threshold
NIL
NIL
0.0
10.0
0.0
8.0
true
false
PENS
"default" 1.0 0 -16777216 true

SLIDER
2
146
166
179
beta1
beta1
0
1
0.9
0.01
1
NIL
HORIZONTAL

SLIDER
2
213
166
246
perror
perror
0
0.1
0
0.001
1
NIL
HORIZONTAL

SLIDER
0
247
168
280
pstraightmove
pstraightmove
0
1
0.1
0.01
1
NIL
HORIZONTAL

SLIDER
-1
280
171
313
egoists
egoists
0
20
0
1
1
NIL
HORIZONTAL

SLIDER
2
113
165
146
alpha2
alpha2
0
1
0.8
0.01
1
NIL
HORIZONTAL

SLIDER
2
179
165
212
beta2
beta2
0
1
0.8
0.01
1
NIL
HORIZONTAL

@#$#@#$#@
This is a Netlogo implementation of the model described in Janssen, M.A. and E. Ostrom (2010), Vulnerability of Social Norms to Incomplete Information, submitted as a chapter for "Dynamic View of Norms" edited by Maria Xenitidou and Bruce Edmonds

Copyright (C) 2010 Marco A. Janssen and Elinor Ostrom

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

link
true
0
Line -7500403 true 150 0 150 300

link direction
true
0
Line -7500403 true 150 150 30 225
Line -7500403 true 150 150 270 225

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 4.1.1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="experimenta" repetitions="100" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <timeLimit steps="1000"/>
    <metric>avgtokens</metric>
    <metric>avgthreshold</metric>
    <metric>vision</metric>
    <metric>alpha1</metric>
    <metric>beta1</metric>
    <metric>perror</metric>
    <metric>prandommove</metric>
    <metric>egoists</metric>
    <enumeratedValueSet variable="perror">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="beta1">
      <value value="0.9"/>
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="egoists">
      <value value="0"/>
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="alpha1">
      <value value="0.9"/>
      <value value="1"/>
    </enumeratedValueSet>
    <steppedValueSet variable="vision" first="1" step="1" last="10"/>
    <enumeratedValueSet variable="prandommove">
      <value value="0.1"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
